pub struct Refined<Inner, Predicate>(pub Inner, pub Predicate);Expand description
Value refinement combinator: filters values through a predicate.
Consistency
inner.consistent(v) && predicate.apply(v).
Tuple Fields
0: Inner1: PredicateTrait Implementations
Sourceimpl<A, Pred> Consistency for Refined<A, Pred>
impl<A, Pred> Consistency for Refined<A, Pred>
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{ self.0.consistent(v) && self.1.apply(v) }Sourcetype Val = <A as Consistency>::Val
type Val = <A as Consistency>::Val
The type of values whose consistency is being checked.
Sourceimpl<Inner, P, T> DerOrd<T> for Refined<Inner, P>
impl<Inner, P, T> DerOrd<T> for Refined<Inner, P>
Sourceproof fn lemma_der_serialize_len(&self, value: T::V)
proof fn lemma_der_serialize_len(&self, value: T::V)
Sourceopen spec fn der_remaining(&self, value: T::V, state: Inner::State) -> Seq<u8>
open spec fn der_remaining(&self, value: T::V, state: Inner::State) -> Seq<u8>
{ self.0.der_remaining(value, state) }Sourceopen spec fn der_state_valid(&self, value: T::V, state: Inner::State) -> bool
open spec fn der_state_valid(&self, value: T::V, state: Inner::State) -> bool
{ self.0.der_state_valid(value, state) }Sourceimpl<A, Pred> EquivSerializers for Refined<A, Pred>
impl<A, Pred> EquivSerializers for Refined<A, Pred>
Sourceimpl<A, Pred> EquivSerializersGeneral for Refined<A, Pred>
impl<A, Pred> EquivSerializersGeneral for Refined<A, Pred>
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{ self.0.equiv_general_inv() }Sourceproof fn lemma_serialize_equiv(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<A, Pred> GoodSerializer for Refined<A, Pred>
impl<A, Pred> GoodSerializer for Refined<A, Pred>
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{ self.0.serialize_inv() }Sourceproof fn lemma_serialize_len(&self, v: Self::SVal)
proof fn lemma_serialize_len(&self, v: Self::SVal)
Sourceimpl<Inner: HasAsn1Start, Predicate: SpecPred<Inner::PVal>> HasAsn1Start for Refined<Inner, Predicate>
Refinement can only narrow an accepted input domain.
impl<Inner: HasAsn1Start, Predicate: SpecPred<Inner::PVal>> HasAsn1Start for Refined<Inner, Predicate>
Refinement can only narrow an accepted input domain.
Sourceopen spec fn asn1_start(&self) -> Asn1StartDomain
open spec fn asn1_start(&self) -> Asn1StartDomain
{ self.0.asn1_start() }Sourceproof fn lemma_parse_implies_asn1_start(&self, input: Seq<u8>)
proof fn lemma_parse_implies_asn1_start(&self, input: Seq<u8>)
Sourceimpl<A, Pred> MinMaxByteLen for Refined<A, Pred>
impl<A, Pred> MinMaxByteLen for Refined<A, Pred>
Sourceimpl<A: NoLookAhead, Pred: SpecPred<A::PVal>> NoLookAhead for Refined<A, Pred>
impl<A: NoLookAhead, Pred: SpecPred<A::PVal>> NoLookAhead for Refined<A, Pred>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{ self.0.no_lookahead_inv() }Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<A: NonMalleable, Pred: SpecPred<A::PVal>> NonMalleable for Refined<A, Pred>
impl<A: NonMalleable, Pred: SpecPred<A::PVal>> NonMalleable for Refined<A, Pred>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{ self.0.nonmal_inv() }Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, Pred> NonTailFmt for Refined<A, Pred>
impl<A, Pred> NonTailFmt for Refined<A, Pred>
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{ self.0.serialize_dps_inv() }Sourceproof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<A: Productive, Pred: SpecPred<A::PVal>> Productive for Refined<A, Pred>
impl<A: Productive, Pred: SpecPred<A::PVal>> Productive for Refined<A, Pred>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ self.0.productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<F, P> Retaggable for Refined<F, P>where
F: Retaggable,
P: Copy,
Allows a value constraint to remain attached when its underlying ASN.1 format is retagged.
impl<F, P> Retaggable for Refined<F, P>where
F: Retaggable,
P: Copy,
Allows a value constraint to remain attached when its underlying ASN.1 format is retagged.
Retagging is delegated to the inner format and the refinement predicate is preserved.
Sourceimpl<A, Pred> SPRoundTripDps for Refined<A, Pred>
impl<A, Pred> SPRoundTripDps for Refined<A, Pred>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{ self.0.unambiguous() }Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<A, Pred> SafeParser for Refined<A, Pred>
impl<A, Pred> SafeParser for Refined<A, Pred>
Sourceimpl<Output: OutputBuf, A, PredFn, T> Serializer<Output, T> for Refined<A, PredFn>where
T: DeepView,
A: Serializer<Output, T>,
PredFn: SpecPred<T::V>,
impl<Output: OutputBuf, A, PredFn, T> Serializer<Output, T> for Refined<A, PredFn>where
T: DeepView,
A: Serializer<Output, T>,
PredFn: SpecPred<T::V>,
Sourceexec fn serialize_into(&self, v: &T, obuf: &mut Output)
exec fn serialize_into(&self, v: &T, obuf: &mut Output)
Sourceimpl<A, Pred> SoundParser for Refined<A, Pred>
impl<A, Pred> SoundParser for Refined<A, Pred>
Sourceimpl<A, Pred> SpecByteLen for Refined<A, Pred>
impl<A, Pred> SpecByteLen for Refined<A, Pred>
Sourceimpl<A, Pred> SpecParser for Refined<A, Pred>
impl<A, Pred> SpecParser for Refined<A, Pred>
Sourceimpl<A, Pred> SpecSerializer for Refined<A, Pred>
impl<A, Pred> SpecSerializer for Refined<A, Pred>
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{ self.0.spec_serialize(v) }Sourcetype SVal = <A as SpecSerializer>::SVal
type SVal = <A as SpecSerializer>::SVal
The type of values to be serialized.
Sourceimpl<A, Pred> SpecSerializerDps for Refined<A, Pred>
impl<A, Pred> SpecSerializerDps for Refined<A, Pred>
Sourceimpl<A, Pred> StaticByteLen for Refined<A, Pred>
impl<A, Pred> StaticByteLen for Refined<A, Pred>
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ A::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
Sourceimpl<A, Pred> ValueByteLen for Refined<A, Pred>
impl<A, Pred> ValueByteLen for Refined<A, Pred>
Sourceopen spec fn value_byte_len(v: Self::T) -> nat
open spec fn value_byte_len(v: Self::T) -> nat
{ A::value_byte_len(v) }Sourceproof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
impl<Inner: Copy, Predicate: Copy> Copy for Refined<Inner, Predicate>